#include <stdlib.h>
#include <stdint.h>
#include <limits.h>

#include <csl.h>
#include <csl_irq.h>
#include <csl_edma.h>

#include "memport.h"
#include "sobel.h"
#include "../common/commdef.h"


extern volatile unsigned read_done;
extern volatile unsigned ext_int;
extern volatile Uint32 *imageptrout;
extern volatile Uint32 wr_num_sec;

  static Int32 output_buffer[NUM_WRITE_SEC * MEMPORT_SECTOR_SIZE / 4];
volatile Uint16 sec_out_clr;

void MemportInit()
{
	//Init interrupts
	IRQ_resetAll();
	IRQ_globalEnable();
	IRQ_nmiEnable();
	IRQ_enable(IRQ_EVT_EDMAINT);
	EDMA_intEnable(EDMA_COMPLETE_CODE);
	
	sec_out_clr = 0;
	wr_num_sec = 0;
  //enable Ext4 interrupt for output sectors 
  (*MEMPORT_READY) = 0xFF0;//FF - nastavi vsechny output sectory jako not ready a 0 nastavi vsechny read sectory jako ready
  (*MEMPORT_MASK)=0xff0;
	IRQ_enable(IRQ_EVT_EXTINT4);

}


interrupt void IsrEdmaInt()
{
  Uint32 i;
  volatile Uint32 sec_num;
  Uint16 mask;
  Int16 *imagebufptr;
  Int16 pokus; 
	uint32_t gie = IRQ_globalDisable();

	IRQ_clear(IRQ_EVT_EDMAINT);
	EDMA_intClear(EDMA_COMPLETE_CODE);

  //if output sector was transfered
  if(sec_out_clr!=0){
  
    //number of transfered sectors
    sec_num = 0;
    mask = 1;
    for(i=0; i<NUM_WRITE_SEC; i++){
      if((sec_out_clr & mask)!=0) sec_num++;
      mask = mask << 1;
    }
    wr_num_sec = wr_num_sec + sec_num;

    //transfer data  
    //imagebufptr = (Int16 *)output_buffer;
    //imagebufptr = (Int16 *)OUTPUT_START;
    /*
    for(i=0; i<(sec_num * MEMPORT_SECTOR_SIZE / 2); i++){
    
  		*imageptrout = (Uint8)(((*imagebufptr)/32));
  		imageptrout++;
  		imagebufptr++;
  		
  	}   
    */
    //clear valid sector after transfer
    (*MEMPORT_READY) = (Uint16)(sec_out_clr << NUM_READ_SEC);
    pokus = (*MEMPORT_READY);
    
    //enable Ext4 interrupt for output sectors 
    (*MEMPORT_MASK)=(Uint16)0x0ff0;
    
    //clear flag
    sec_out_clr = 0;
    
 		IRQ_enable(IRQ_EVT_EXTINT4);
 		

  }
  

	read_done=1;

	IRQ_globalRestore(gie);
}

interrupt void IsrExtInt4()
{
	
	Uint16 mask;
	Uint32 sec_ready;
  Uint32 sec_in_ready, sec_in_num, sec_in_off, sec_in_len;
  Uint32 sec_out_ready, sec_out_num, sec_out_off, sec_out_len;
  int i;
  Uint32 pokus;
	
	uint32_t gie = IRQ_globalDisable();
	IRQ_clear(IRQ_EVT_EXTINT4);
	
	
	//read ready_reg status (first valid sec, nuber of continuoius valid sec)
	sec_ready = (*MEMPORT_READY);// & (*MEMPORT_MASK);
  sec_out_ready = (sec_ready & 0xff0) >> NUM_READ_SEC;
  sec_in_ready = sec_ready & 0xf;

  sec_out_num = 0;
  sec_out_off = 0;
  mask=1;
  for(i=0; i<NUM_WRITE_SEC; i++){
    if((sec_out_ready & mask) !=0){
      //first valid sector
      if(sec_out_num==0) sec_out_off = i;
      //increment sector number
      sec_out_num++;
    } else {
      //stop counting continuos valid sectors
      if (sec_out_num!=0) mask = 0;//break;
    }
    mask = mask << 1;
  }

	//setup EDMA (set address to firs valid sector, block length = number od continuous valid sec * sec_size)
	//set address to first valid sector
  (*MEMPORT_ADDR)=((NUM_READ_SEC + sec_out_off) * MEMPORT_SECTOR_SIZE / 2);
  pokus = (*MEMPORT_ADDR);
  //block length
  sec_out_len = sec_out_num * MEMPORT_SECTOR_SIZE / 4;
  //setup EDMA transfer

  EDMA_qdmaConfigArgs(
    0x40380001,
  	EDMA_SRC_OF(MEMPORT_DATA),
  	EDMA_CNT_OF(sec_out_len),
  	EDMA_DST_OF(imageptrout),
  	EDMA_IDX_OF(0x00000004)
  );	    	
	imageptrout += sec_out_len;
	//mask output valid reg to prevent IsrExtInt4 early call  
	(*MEMPORT_MASK)=0;
	
	//set flag for edma complete event ISR to transform data to output
	mask=0;
	for(i = 0; i < sec_out_num; i++){
	  mask = mask << 1;
    mask |= 0x1; 
  }
  mask = mask << sec_out_off;
  //set mask to clear valid sector after transfer 
	sec_out_clr = mask;
  
  ext_int = 1;

	IRQ_disable(IRQ_EVT_EXTINT4);
	IRQ_globalRestore(gie);
}

